@@ -29,9 +29,9 @@ from click.core import ParameterSource
 from geedim import schema, Initialize, version
 from geedim.collection import MaskedCollection
 from geedim.download import BaseImage, supported_dtypes
-from geedim.enums import CloudMaskMethod, CompositeMethod, ResamplingMethod
+from geedim.enums import CloudMaskMethod, CompositeMethod, ResamplingMethod, ExportType
 from geedim.mask import MaskedImage
-from geedim.utils import get_bounds, Spinner
+from geedim.utils import get_bounds, Spinner, asset_id
 from rasterio.errors import CRSError
 
 logger = logging.getLogger(__name__)
@@ -96,7 +96,7 @@ class ChainedCommand(click.Command):
         if ('like' in ctx.params) and (ctx.params['like'] is not None):
             # populate crs, crs_transform & shape parameters from a template raster
             with rio.open(ctx.params['like'], 'r') as im:
-                ctx.params['crs'] = f'EPSG:{im.crs.to_epsg()}'  # TODO: WKT?
+                ctx.params['crs'] = im.crs.to_string()
                 ctx.params['crs_transform'] = im.transform
                 ctx.params['shape'] = im.shape
 
@@ -181,6 +181,11 @@ def _comp_method_cb(ctx, param, value):
     return CompositeMethod(value) if value else None
 
 
+def _export_type_cb(ctx, param, value):
+    """click callback to convert export type string to enum."""
+    return ExportType(value)
+
+
 def _prepare_image_list(obj: SimpleNamespace, mask=False) -> List[MaskedImage, ]:
     """Validate and prepare the obj.image_list for export/download.  Returns a list of MaskedImage objects."""
     if len(obj.image_list) == 0:
@@ -390,13 +395,18 @@ cli.add_command(config)
     help='Region defined by geojson polygon or raster file. Use "-" to read geojson from stdin.'
 )
 @click.option(
-    '-fp', '--fill-portion', type=click.FloatRange(min=0, max=100), default=0, show_default=True,
-    help='Lower limit on the filled portion of the region (%).'
+    '-fp', '--fill-portion', '--fill', metavar='VALUE', type=click.FloatRange(min=0, max=100), default=None,
+    is_flag=False, flag_value=0,
+    show_default='don\'t calculate, or filter on, fill portion',
+    help='Lower limit on the portion of the region that contains filled/valid image pixels (%).  Uses zero if VALUE is '
+         'not specified.'
 )
 @click.option(
-    '-cp', '--cloudless-portion', type=click.FloatRange(min=0, max=100), default=0, show_default=True,
-    help='Lower limit on the cloud/shadow free portion of the region (%).  If cloud/shadow masking is not supported '
-    'for the specified collection, ``--cloudless-portion`` will operate like ``--fill-portion``.'
+    '-cp', '--cloudless-portion', '--cloudless', metavar='VALUE', type=click.FloatRange(min=0, max=100), default=None,
+    is_flag=False, flag_value=0, show_default='don\'t calculate, or filter on, cloudless portion',
+    help='Lower limit on the portion of filled pixels that are cloud/shadow free (%).  Uses zero if VALUE is not '
+         'specified.  If cloud/shadow masking is not supported for the specified collection, ``--cloudless-portion`` '
+         'has no effect.'
 )
 @click.option(
     '-cf', '--custom-filter', type=click.STRING, default=None,
@@ -419,8 +429,9 @@ def search(
     """
     Search for images.
 
-    Search a Google Earth Engine image collection for images, based on date, region, portion of filled pixels, and
-    custom filters.  Cloud/shadow-free (cloudless) portion filtering is supported on the following collections:
+    Search a Google Earth Engine image collection for images, based on date, region, portion of filled pixels in
+    region, and custom filters.  Filtering on cloud/shadow-free (cloudless) portion of filled pixels is supported on
+    the following collections:
     \b
 
         ===========  ===========================
@@ -439,8 +450,12 @@ def search(
 
     The search must be filtered with at least one of the ``--start-date``, ``--bbox`` or ``--region`` options.
 
-    Note that filled/cloudless portions are not taken from the granule metadata, but are calculated as portions of the
-    specified search region for improved accuracy.
+    Note that filled/cloudless portions are not taken from the granule metadata, but are calculated as portions
+    inside the specified search region for improved accuracy.  These portions are only found and reported when one or
+    both of ``--fill-portion`` / ``--cloudless-portion`` are specified.
+
+    Search speed can be improved by specifying ``--custom-filter``, and or by omitting ``--fill-portion`` /
+    ``--cloudless-portion``.
     \b
 
     Examples
@@ -450,9 +465,9 @@ def search(
 
         geedim search -c LARSE/GEDI/GEDI02_A_002_MONTHLY -s 2021-12-01 -e 2022-02-01 --bbox 23 -34 23.2 -33.8 --fill-portion 0.5
 
-    Search the Landsat-9 collection for images with a cloud/shadow free portion of at least 50%::
+    Search the Landsat-9 collection for images, reporting the cloud/shadow free portion::
 
-        geedim search -c l9-c2-l2 -s 2022-01-01 -e 2022-03-01 --bbox 23 -34 23.2 -33.8 --cloudless-portion 50
+        geedim search -c l9-c2-l2 -s 2022-01-01 -e 2022-03-01 --bbox 23 -34 23.2 -33.8 --cloudless-portion
 
     Search the Landsat-8 collection for images whose `CLOUD_COVER_LAND` property is less than 50%, and include the
     `CLOUD_COVER_LAND`, and `CLOUD_COVER` image properties in the search results::
@@ -476,7 +491,7 @@ def search(
             custom_filter=custom_filter, **obj.cloud_kwargs
         )
         # retrieve search result properties from EE
-        num_images = gd_collection.properties
+        num_images = len(gd_collection.properties)
 
     if num_images == 0:
         logger.info('No images found\n')
@@ -507,6 +522,10 @@ cli.add_command(search)
 @shape_option
 @like_option
 @dtype_option
+@click.option(
+    '-bn', '--band-name', 'bands', type=click.STRING, multiple=True, default=None, show_default='all bands',
+    help='Band name(s) to download.'
+)
 @mask_option
 @resampling_option
 @scale_offset_option
@@ -535,9 +554,10 @@ def download(obj, image_id, bbox, region, like, download_dir, mask, max_tile_siz
     and downloaded as separate tiles, then re-assembled into a single GeoTIFF.  Downloaded image files are populated
     with metadata from the Earth Engine image and STAC.
 
-    This command can be chained after the ``composite`` command, to download the composite image.  It can also be
-    chained after the ``search`` command, in which case the search result images will be downloaded, without the need
-    to specify image IDs with ``--id``, or region with ``--bbox`` / ``--region``.
+    This command can be chained after the ``composite`` command, to download the composite image, or it can be
+    chained after an asset ``export`` to download the asset image.  It can also be chained after the ``search`` command,
+    in which case the search result images will be downloaded, without the need to specify image IDs with ``--id``, or
+    region with ``--bbox`` / ``--region``.
 
     The following auxiliary bands are added to images from collections with support for cloud/shadow masking:
     \b
@@ -594,6 +614,16 @@ cli.add_command(download)
 # export command
 @click.command(cls=ChainedCommand)
 @click.option('-i', '--id', 'image_id', type=click.STRING, multiple=True, help='Earth Engine image ID(s) to export.')
+@click.option(
+    '-t', '--type', type=click.Choice([t.value for t in ExportType], case_sensitive=True),
+    default=BaseImage._default_export_type.value, show_default=True, callback=_export_type_cb,
+    help='Export type.'
+)
+@click.option(
+    '-f', '-df', '--folder', '--drive-folder', type=click.STRING, default=None,
+    help='Google Drive folder, Earth Engine asset project, or Google Cloud Storage bucket to export image(s) to.  '
+         'Interpretation based on :option:`--type`.'
+)
 @crs_option
 @bbox_option
 @region_option
@@ -602,24 +632,24 @@ cli.add_command(download)
 @shape_option
 @like_option
 @dtype_option
+@click.option(
+    '-bn', '--band-name', 'bands', type=click.STRING, multiple=True, default=None, show_default='all bands',
+    help='Band name(s) to download.'
+)
 @mask_option
 @resampling_option
 @scale_offset_option
-@click.option(
-    '-df', '--drive-folder', type=click.STRING, default=None, show_default='root folder.',
-    help='Google Drive folder to export image(s) to.'
-)
 @click.option(
     '-w/-nw', '--wait/--no-wait', default=True, show_default=True, help='Whether to wait for the export to complete.'
 )
 @click.pass_obj
-def export(obj, image_id, bbox, region, like, drive_folder, mask, wait, **kwargs):
+def export(obj, image_id, type, folder, bbox, region, like, mask, wait, **kwargs):
     # @formatter:off
     """
-    Export image(s) to Google Drive.
+    Export image(s).
 
-    Export Earth Engine image(s) to GeoTIFF file(s) on Google Drive, allowing optional region of interest,
-    and other image formatting options to be specified.
+    Export Earth Engine image(s) to Google Drive, Earth Engine asset, or Google Cloud Storage, allowing optional region
+    of interest, and other image formatting options to be specified.
 
     This command can be chained after the ``composite`` command, to export the composite image.  It can also be
     chained after the ``search`` command, in which case the search result images will be exported, without the need
@@ -648,32 +678,40 @@ def export(obj, image_id, bbox, region, like, drive_folder, mask, wait, **kwargs
     When ``--crs``, ``--scale``, ``--crs-transform`` and ``--shape`` are not specified, the pixel grids of the
     exported and Earth Engine images will coincide.
 
-    Image filenames are derived from their Earth Engine ID.
+    Image file or asset names are derived from their Earth Engine ID.
     \b
 
     Examples
     --------
 
-    Export a region of a Landsat-9 image, applying the cloud/shadow mask and converting to uint16::
+    Export a region of a Landsat-9 image to an Earth Engine asset, applying the cloud/shadow mask and converting to
+    uint16::
 
-        geedim export -i LANDSAT/LC09/C02/T1_L2/LC09_173083_20220308 --mask --bbox 21.6 -33.5 21.7 -33.4 --dtype uint16
+        geedim export -i LANDSAT/LC09/C02/T1_L2/LC09_173083_20220308 --type asset --folder <your cloud project> --mask --bbox 21.6 -33.5 21.7 -33.4 --dtype uint16
 
-    Export the results of a MODIS NBAR search to the 'geedim' folder, specifying a CRS and scale to reproject to::
+    Export the results of a MODIS NBAR search to Google Drive in the 'geedim' folder, specifying a CRS and scale to
+    reproject to::
 
         geedim search -c MODIS/006/MCD43A4 -s 2022-01-01 -e 2022-01-03 --bbox 23 -34 24 -33 export --crs EPSG:3857 --scale 500 -df geedim
     """
     # @formatter:on
     logger.info('\nExporting:\n')
+    if (type == ExportType.asset) and not folder:
+        raise click.BadParameter('--folder must be specified when exporting to asset', param_hint='--folder')
     image_list = _prepare_image_list(obj, mask=mask)
     export_tasks = []
     for im in image_list:
-        task = im.export(im.name, folder=drive_folder, wait=False, region=obj.region, **kwargs)
+        task = im.export(im.name, type=type, folder=folder, wait=False, region=obj.region, **kwargs)
         export_tasks.append(task)
         logger.info(f'Started {im.name}') if not wait else None
 
     if wait:
-        for task in export_tasks:
+        obj.image_list = [] if type == ExportType.asset else obj.image_list
+        for task, im in zip(export_tasks, image_list):
             BaseImage.monitor_export(task)
+            if type == ExportType.asset:
+                # add asset ids, so that assets can be downloaded or composited with chained commands
+                obj.image_list += [asset_id(im.name, folder)]
 
 
 cli.add_command(export)
@@ -743,16 +781,17 @@ def composite(obj, image_id, mask, method, resampling, bbox, region, date):
                     | pixel has the same cloud distance, the first one in the
                     | stack is selected.
         `mosaic`    Use the first unmasked pixel in the stack.
-        `medoid`    | Use the medoid of the unmasked pixels i.e. the pixels
-                    | of the image with minimum summed difference (across
-                    | bands) to the median over the input images.
+        `medoid`    | Use the medoid of the unmasked pixels i.e. the pixel
+                    | of the image with the minimum sum of spectral distances
+                    | to the rest of the input images.
                     | Maintains relationship between bands.
         `median`    Use the median of the unmasked pixels.
         `mode`      Use the mode of the unmasked pixels.
         `mean`      Use the mean of the unmasked pixels.
         ==========  ========================================================
 
-    For the `mosaic` and `q-mosaic` methods there are three ways of ordering (i.e. prioritising) images in the stack:
+    For the `mosaic`, `q-mosaic` and `medoid` methods there are three ways of ordering (i.e. prioritising) images in
+    the stack:
     \b
 
         * | If ``--date`` is specified, images are sorted by the absolute
