@@ -37,7 +37,7 @@ from tqdm.auto import tqdm
 from tqdm.contrib.logging import logging_redirect_tqdm
 
 from geedim import utils
-from geedim.enums import ResamplingMethod
+from geedim.enums import ResamplingMethod, ExportType
 from geedim.stac import StacCatalog, StacItem
 from geedim.tile import Tile
 
@@ -58,6 +58,7 @@ class BaseImage:
     _default_resampling = ResamplingMethod.near
     _ee_max_tile_size = 32
     _ee_max_tile_dim = 10000
+    _default_export_type = ExportType.drive
 
     def __init__(self, ee_image: ee.Image):
         """
@@ -167,6 +168,15 @@ class BaseImage:
         """ Geojson polygon of the image extent.  None if the image is a composite. """
         if ('properties' not in self._ee_info) or ('system:footprint' not in self._ee_info['properties']):
             return None
+        footprint = self._ee_info['properties']['system:footprint']
+
+        if ('type' in footprint) and ('coordinates' in footprint) and footprint['type'] == 'LinearRing':
+            # convert LinearRing to simple Polygon
+            # this is necessary to make footprint compatible as a region in other geedim methods e.g.
+            # MaskedCollection.search or BaseImage.download.
+            footprint['coordinates'] = [footprint['coordinates']]
+            footprint['type'] = 'Polygon'
+
         return self._ee_info['properties']['system:footprint']
 
     @property
@@ -229,6 +239,13 @@ class BaseImage:
             count=self.count
         )
 
+    @property
+    def bounded(self) -> bool:
+        """ True if the image is bounded, otherwise False. """
+        # TODO: an unbounded region could also have these bounds
+        unbounded_bounds = (-180, -90, 180, 90)
+        return (self.footprint is not None) and (features.bounds(self.footprint) != unbounded_bounds)
+
     @staticmethod
     def _get_projection(ee_info: Dict, min_scale=True) -> Dict:
         """
@@ -374,7 +391,7 @@ class BaseImage:
     def _prepare_for_export(
         self, crs: str = None, crs_transform: Tuple[float] = None, shape: Tuple[int, int] = None, region: Dict =
         None, scale: float = None, resampling: ResamplingMethod = _default_resampling, dtype: str = None,
-        scale_offset: bool = False
+        scale_offset: bool = False, bands: List[str] = None,
     ) -> 'BaseImage':
         """
         Prepare the encapsulated image for export/download.  Will reproject, resample, clip and convert the image
@@ -384,14 +401,13 @@ class BaseImage:
         and ``shape``.  If no bounds are specified (with either ``region``, or ``crs_transform`` & ``shape``), the
         entire image granule is exported.
 
-        When ``crs`` and ``scale`` are not specified (or match those of the encapsulated image), and ``crs_transform``
-        & ``shape`` are not specified, the pixel grid of the exported image will coincide with that of the encapsulated
-        image.
+        When ``crs``, ``scale``, ``crs_transform`` & ``shape`` are not specified, the pixel grid of the exported
+        image will coincide with that of the encapsulated image.
 
         Parameters
         ----------
         crs : str, optional
-            WKT, EPSG etc. specification of CRS to export to.  Where image bands have different CRSs, all are
+            WKT or EPSG specification of CRS to export to.  Where image bands have different CRSs, all are
             re-projected to this CRS. Defaults to use the CRS of the minimum scale band if available.
         crs_transform: tuple of float, list of float, rio.Affine, optional
             List of 6 numbers specifying an affine transform in the specified CRS.  In row-major order:
@@ -431,7 +447,7 @@ class BaseImage:
                     f'crs, crs_transform & shape.'
                 )
 
-        if (not self.footprint) and (not region and (not crs or not crs_transform or not shape)):
+        if (not region and (not crs or not crs_transform or not shape)) and (not self.bounded):
             # if the image has no footprint (i.e. it is 'unbounded'), either region; or crs, crs_transform and shape
             # must be specified
             raise ValueError(
@@ -439,63 +455,67 @@ class BaseImage:
                 f'shape.'
             )
 
-        if self.crs == 'EPSG:4326' and not scale and not shape:
+        if not scale and not shape and self.crs == 'EPSG:4326':
             # If the image is in EPSG:4326, either scale (in meters); or shape must be specified.
             # Note that ee.Image.prepare_for_export() expects a scale in meters, but if the image is EPSG:4326,
             # the default scale is in degrees.
             raise ValueError(f'This image is in EPSG:4326, you need to specify a scale (in meters); or a shape.')
 
-        if crs == 'SR-ORG:6974':
-            raise ValueError(
-                'There is an earth engine bug exporting in SR-ORG:6974, specify another CRS: '
-                'https://issuetracker.google.com/issues/194561313'
-            )
-
         if scale and shape:
             # This error is raised in later calls to ee.Image.getInfo(), but is neater to raise here first
             raise ValueError('You can specify one of scale or shape, but not both.')
 
+        if bands:
+            band_diff = set(bands).difference([band_prop['name'] for band_prop in self.band_properties])
+            if len(band_diff) > 0:
+                # If one or more specified bands don't exist, raise an error
+                raise ValueError(f'The band(s) {list(band_diff)} don\'t exist.')
+
+        # Create a new BaseImage of band subset is specified.  This is done here so that crs, scale etc
+        # parameters used below will have the values specific to bands.
+        exp_image = BaseImage(self.ee_image.select(bands)) if bands else self
+
         # perform image scale/offset, dtype and resampling operations
-        ee_image = self.ee_image
+        ee_image = exp_image.ee_image
         if scale_offset:
-            ee_image = self._scale_offset(ee_image, self.band_properties)
+            ee_image = BaseImage._scale_offset(ee_image, exp_image.band_properties)
             im_dtype = 'float64'
         else:
-            im_dtype = self.dtype
+            im_dtype = exp_image.dtype
 
         resampling = ResamplingMethod(resampling)
-        if resampling != self._default_resampling:
-            if not self.has_fixed_projection:
+        if resampling != BaseImage._default_resampling:
+            if not exp_image.has_fixed_projection:
                 raise ValueError(
                     'This image has no fixed projection and cannot be resampled.  If this image is a composite, '
-                    'you can resample the images used to create the composite.'
+                    'you can resample the component images used to create the composite.'
                 )
             ee_image = utils.resample(ee_image, resampling)
-
-        ee_image = self._convert_dtype(ee_image, dtype=dtype or im_dtype)
+        # TODO: only do this if there is a change?  Or do we need to standardise the bands?
+        ee_image = BaseImage._convert_dtype(ee_image, dtype=dtype or im_dtype)
 
         # configure the export parameter values
-        crs = crs or self.crs
+        crs = crs or exp_image.crs
         if not crs_transform and not shape:
             # if none of crs_transform, shape, region or scale are specified, then set region and scale to defaults
-            region = region or self.footprint
-            scale = scale or self.scale
+            region = region or exp_image.footprint
+            scale = scale or exp_image.scale
 
-            if (crs == self.crs) and (scale == self.scale):
+            if (crs == exp_image.crs) and (scale == exp_image.scale):
                 # if crs_transform & shape are not already specified, and crs & scale match this image's crs & scale,
-                # then set crs_transform and shape to export on this image's (self) pixel grid
-                if region == self.footprint:
-                    crs_transform = self.transform
-                    shape = self.shape
+                # then set crs_transform and shape to export on export image's pixel grid
+                if region == exp_image.footprint:
+                    crs_transform = exp_image.transform
+                    shape = exp_image.shape
                 else:
                     # find a crs_transform and shape that encompasses region
                     if isinstance(region, ee.Geometry):
                         region = region.getInfo()
                     region_crs = region['crs']['properties']['name'] if 'crs' in region else 'EPSG:4326'
-                    region_bounds = warp.transform_bounds(region_crs, crs, *features.bounds(region))
-                    region_win = windows.from_bounds(*region_bounds, transform=self.transform)
+                    region_bounds = warp.transform_bounds(region_crs, utils.rio_crs(crs), *features.bounds(region))
+                    region_win = windows.from_bounds(*region_bounds, transform=exp_image.transform)
                     region_win = utils.expand_window_to_grid(region_win)
-                    crs_transform = self.transform * rio.Affine.translation(region_win.col_off, region_win.row_off)
+                    crs_transform = exp_image.transform * rio.Affine.translation(region_win.col_off, region_win.row_off)
                     shape = (region_win.height, region_win.width)
 
                 # prevent exporting with region & scale now that crs_transform and shape are set
@@ -506,12 +526,13 @@ class BaseImage:
         crs_transform = crs_transform[:6] if crs_transform else None
         dimensions = shape[::-1] if shape else None
         export_kwargs = dict(
-            crs=crs, crs_transform=crs_transform, dimensions=dimensions, region=region, scale=scale,
+            crs=crs, crs_transform=crs_transform, dimensions=dimensions, region=region, scale=scale, bands=bands,
             fileFormat='GeoTIFF', filePerBand=False
         )
         # drop items with values==None
         export_kwargs = {k: v for k, v in export_kwargs.items() if v is not None}
         logger.debug(f'ee.Image.prepare_for_export() params: {export_kwargs}')
+        # TODO: prepare_for_export does not seem to be used in ee internals any more
         ee_image, _ = ee_image.prepare_for_export(export_kwargs)
         return BaseImage(ee_image)
 
@@ -538,21 +559,20 @@ class BaseImage:
         nodata = nodata_dict[exp_image.dtype] if set_nodata else None
         profile = dict(
             driver='GTiff', dtype=exp_image.dtype, nodata=nodata, width=exp_image.shape[1], height=exp_image.shape[0],
-            count=exp_image.count, crs=CRS.from_string(exp_image.crs), transform=exp_image.transform,
-            compress='deflate', interleave='band', tiled=True, photometric=None,
+            count=exp_image.count, crs=CRS.from_string(utils.rio_crs(exp_image.crs)), transform=exp_image.transform,
+            compress='deflate', interleave='band', tiled=True, photometric='MINISBLACK',
         )
         # add BIGTIFF support if the uncompressed image is bigger than 4GB
         if exp_image.size >= 4e9:
             profile.update(bigtiff=True)
         return exp_image, profile
 
-    @staticmethod
     def _get_tile_shape(
-        exp_image: 'BaseImage', max_tile_size: Optional[float] = None, max_tile_dim: Optional[int] = None
+        self, max_tile_size: Optional[float] = None, max_tile_dim: Optional[int] = None
     ) -> Tuple[Tuple[int, int], int]:  # yapf: disable
         """
-        Return a tile shape and number of tiles for a given BaseImage, such that the tile shape satisfies GEE
-        download limits, and is 'square-ish'.
+        Return a tile shape and number of tiles, such that the tile shape satisfies GEE download limits, and is
+        'square-ish'.
         """
         # convert max_tile_size from MB to bytes & set to EE default if None
         if max_tile_size and (max_tile_size > BaseImage._ee_max_tile_size):
@@ -569,15 +589,15 @@ class BaseImage:
         max_tile_dim = max_tile_dim or BaseImage._ee_max_tile_dim   # set max_tile_dim to EE default if None
 
         # find the total number of tiles the image must be divided into to satisfy max_tile_size
-        image_shape = np.array(exp_image.shape, dtype='int64')
-        dtype_size = np.dtype(exp_image.dtype).itemsize
-        image_size = exp_image.size
-        if exp_image.dtype.endswith('int8'):
+        image_shape = np.array(self.shape, dtype='int64')
+        dtype_size = np.dtype(self.dtype).itemsize
+        image_size = self.size
+        if self.dtype.endswith('int8'):
             # workaround for GEE overestimate of *int8 dtype download sizes
             dtype_size *= 2
             image_size *= 2
 
-        pixel_size = dtype_size * exp_image.count
+        pixel_size = dtype_size * self.count
 
         num_tile_shape = np.array([1, 1], dtype='int64')
         tile_size = image_size
@@ -593,19 +613,25 @@ class BaseImage:
         tile_shape = tuple(tile_shape.tolist())
         return tile_shape, num_tiles
 
-    @staticmethod
-    def _build_overviews(dataset: rio.io.DatasetWriter, max_num_levels: int = 8, min_ovw_pixels: int = 256):
-        """ Build internal overviews, downsampled by successive powers of 2, for an open rasterio dataset. """
-        if dataset.closed:
-            raise IOError('Image dataset is closed')
+    def _build_overviews(self, filename: Union[str, pathlib.Path], max_num_levels: int = 8, min_ovw_pixels: int = 256):
+        """ Build internal overviews, downsampled by successive powers of 2, for a given filename. """
 
-        # limit overviews so that the highest level has at least 2**8=256 pixels along the shortest dimension,
-        # and so there are no more than 8 levels.
-        max_ovw_levels = int(np.min(np.log2(dataset.shape)))
-        min_level_shape_pow2 = int(np.log2(min_ovw_pixels))
-        num_ovw_levels = np.min([max_num_levels, max_ovw_levels - min_level_shape_pow2])
-        ovw_levels = [2**m for m in range(1, num_ovw_levels + 1)]
-        dataset.build_overviews(ovw_levels, RioResampling.average)
+        # TODO: revisit multi-threaded overviews on gdal update
+        # build overviews in a single threaded environment (currently gdal reports errors when building overviews
+        # with GDAL_NUM_THREADS='ALL_CPUs' - see https://github.com/OSGeo/gdal/issues/7921)
+        env_dict = dict(GTIFF_FORCE_RGBA=False, COMPRESS_OVERVIEW='DEFLATE')
+        if self.size >= 4e9:
+            env_dict.update(BIGTIFF_OVERVIEW=True)
+
+        with rio.Env(**env_dict), rio.open(filename, 'r+') as ds:
+            # limit overviews so that the highest level has at least 2**8=256 pixels along the shortest dimension,
+            # and so there are no more than 8 levels.
+            max_ovw_levels = int(np.min(np.log2(ds.shape)))
+            min_level_shape_pow2 = int(np.log2(min_ovw_pixels))
+            num_ovw_levels = np.min([max_num_levels, max_ovw_levels - min_level_shape_pow2])
+            ovw_levels = [2**m for m in range(1, num_ovw_levels + 1)]
+            with utils.suppress_rio_logs(logging.CRITICAL):
+                ds.build_overviews(ovw_levels, RioResampling.average)
 
     def _write_metadata(self, dataset: rio.io.DatasetWriter):
         """ Write Earth Engine and STAC metadata to an open rasterio dataset. """
@@ -637,12 +663,11 @@ class BaseImage:
                 dataset.set_band_description(band_i + 1, clean_band_dict['name'])
             dataset.update_tags(band_i + 1, **clean_band_dict)
 
-    @staticmethod
-    def _tiles(exp_image: 'BaseImage', tile_shape: Tuple[int, int]) -> Iterator[Tile]:
+    def _tiles(self, tile_shape: Tuple[int, int]) -> Iterator[Tile]:
         """
         Iterator over downloadable image tiles.
 
-        Divides an image into adjoining tiles no bigger than `tile_shape`.
+        Divides the image into adjoining tiles no bigger than `tile_shape`.
 
         Parameters
         ----------
@@ -659,13 +684,13 @@ class BaseImage:
         """
 
         # split the image up into tiles of at most `tile_shape` dimension
-        image_shape = exp_image.shape
+        image_shape = self.shape
         start_range = product(range(0, image_shape[0], tile_shape[0]), range(0, image_shape[1], tile_shape[1]))
         for tile_start in start_range:
             tile_stop = np.clip(np.add(tile_start, tile_shape), a_min=None, a_max=image_shape)
             clip_tile_shape = (tile_stop - tile_start).tolist()  # tolist is just to convert to native int
             tile_window = windows.Window(tile_start[1], tile_start[0], clip_tile_shape[1], clip_tile_shape[0])
-            yield Tile(exp_image, tile_window)
+            yield Tile(self, tile_window)
 
     @staticmethod
     def monitor_export(task: ee.batch.Task, label: str = None):
@@ -688,9 +713,13 @@ class BaseImage:
 
         # poll EE until the export preparation is complete
         with utils.Spinner(label=f'Preparing {label}: ', leave='done'):
-            while 'progress' not in status['metadata']:
+            while ('done' not in status) or (not status['done']):
                 time.sleep(5 * pause)
                 status = ee.data.getOperation(task.name)
+                if 'progress' in status['metadata']:
+                    break
+                elif status['metadata']['state'] == 'FAILED':
+                    raise IOError(f"Export failed \n{status}")
 
         # wait for export to complete, displaying a progress bar
         bar_format = '{desc}: |{bar}| [{percentage:5.1f}%] in {elapsed:>5s} (eta: {remaining:>5s})'
@@ -705,28 +734,35 @@ class BaseImage:
             else:
                 raise IOError(f"Export failed \n{status}")
 
-    def export(self, filename: str, folder: str = None, wait: bool = True, **kwargs) -> ee.batch.Task:
+    def export(
+        self, filename: str, type: ExportType = _default_export_type, folder: str = None, wait: bool = True, **kwargs
+    ) -> ee.batch.Task:
         """
-        Export the encapsulated image to Google Drive.
+        Export the encapsulated image to Google Drive, Earth Engine asset or Google Cloud Storage.
 
         Export bounds and resolution can be specified with ``region`` and ``scale`` / ``shape``, or ``crs_transform``
         and ``shape``.  If no bounds are specified (with either ``region``, or ``crs_transform`` & ``shape``), the
         entire image granule is exported.
 
-        When ``crs`` and ``scale`` are not specified (or match those of the encapsulated image), and ``crs_transform``
-        & ``shape`` are not specified, the pixel grid of the exported image will coincide with that of the encapsulated
-        image.
+        When ``crs``, ``scale``, ``crs_transform`` & ``shape`` are not specified, the pixel grid of the exported
+        image will coincide with that of the encapsulated image.
 
         Parameters
         ----------
         filename : str
-            Name of the task and destination file.
+            Name of the export task, and destination file or asset name.
+        type : ExportType, optional
+            Export type.
         folder : str, optional
-            Google Drive folder to export to. Defaults to the root.
+            Google Drive folder (when ``type`` is :attr:`~geedim.enums.ExportType.drive`),
+            Earth Engine asset project (when ``type`` is :attr:`~geedim.enums.ExportType.asset`),
+            or Google Cloud Storage bucket (when ``type`` is :attr:`~geedim.enums.ExportType.cloud`), to export to.
+            If ``type`` is :attr:`~geedim.enums.ExportType.asset` and ``folder`` is not specified, ``filename``
+            should be a valid Earth Engine asset ID.
         wait : bool
             Wait for the export to complete before returning.
         crs : str, optional
-            WKT, EPSG etc specification of CRS to export to.  Where image bands have different CRSs, all are
+            WKT or EPSG specification of CRS to export to.  Where image bands have different CRSs, all are
             re-projected to this CRS. Defaults to use the CRS of the minimum scale band if available.
         crs_transform: tuple of float, list of float, rio.Affine, optional
             List of 6 numbers specifying an affine transform in the specified CRS.  In row-major order:
@@ -747,6 +783,8 @@ class BaseImage:
            `float64`). Defaults to auto select a minimal type that can represent the range of pixel values.
         scale_offset: bool, optional
             Whether to apply any EE band scales and offsets to the image.
+        bands: list of str, optional
+            List of band names to export.
 
         Returns
         -------
@@ -755,17 +793,40 @@ class BaseImage:
         """
 
         exp_image = self._prepare_for_export(**kwargs)
+        if exp_image.crs == 'SR-ORG:6974':
+            logger.warning(
+                'There is an earth engine bug exporting in SR-ORG:6974, you will need to edit the exported file to '
+                'replace the CRS with SR-ORG:6842. See: https://issuetracker.google.com/issues/194561313.'
+            )
 
         if logger.getEffectiveLevel() <= logging.DEBUG:
-            logger.debug(f'Uncompressed size: {self._str_format_size(exp_image.size)}')
+            logger.debug(f'Uncompressed size: {BaseImage._str_format_size(exp_image.size)}')
 
         # create export task and start
-        task = ee.batch.Export.image.toDrive(
-            image=exp_image.ee_image, description=filename[:100], folder=folder, fileNamePrefix=filename, maxPixels=1e9
-        )
+        type = ExportType(type)
+        if type == ExportType.drive:
+            task = ee.batch.Export.image.toDrive(
+                image=exp_image.ee_image, description=filename[:100], folder=folder, fileNamePrefix=filename,
+                maxPixels=1e9, formatOptions=dict(cloudOptimized=True),
+            )
+        elif type == ExportType.asset:
+            # if folder is specified create an EE asset ID from it and filename,
+            # else assume filename is a valid EE asset ID
+            asset_id = utils.asset_id(filename, folder) if folder else filename
+            # fix description for when filename is asset id with forward slashes
+            description = filename.replace('/', '-')[:100]
+            task = ee.batch.Export.image.toAsset(
+                image=exp_image.ee_image, description=description, assetId=asset_id, maxPixels=1e9,
+            )
+        else:
+            task = ee.batch.Export.image.toCloudStorage(
+                image=exp_image.ee_image, description=filename[:100], bucket=folder, maxPixels=1e9,
+                formatOptions=dict(cloudOptimized=True),
+            )
+
         task.start()
         if wait:  # wait for completion
-            self.monitor_export(task)
+            BaseImage.monitor_export(task)
         return task
 
     def download(
@@ -784,9 +845,8 @@ class BaseImage:
         and ``shape``.  If no bounds are specified (with either ``region``, or ``crs_transform`` & ``shape``), the
         entire image granule is downloaded.
 
-        When ``crs`` and ``scale`` are not specified (or match those of the encapsulated image), and ``crs_transform``
-        & ``shape`` are not specified, the pixel grid of the downloaded image will coincide with that of the
-        encapsulated image.
+        When ``crs``, ``scale``, ``crs_transform`` & ``shape`` are not specified, the pixel grid of the exported
+        image will coincide with that of the encapsulated image.
 
         Parameters
         ----------
@@ -801,7 +861,7 @@ class BaseImage:
         max_tile_dim: int, optional
             Maximum tile width/height (pixels).  If None, defaults to Earth Engine download limit (10000).
         crs : str, optional
-            WKT, EPSG etc. specification of CRS to export to.  Where image bands have different CRSs, all are
+            WKT or EPSG specification of CRS to export to.  Where image bands have different CRSs, all are
             re-projected to this CRS. Defaults to use the CRS of the minimum scale band if available.
         crs_transform: tuple of float, list of float, rio.Affine, optional
             List of 6 numbers specifying an affine transform in the specified CRS.  In row-major order:
@@ -822,6 +882,8 @@ class BaseImage:
            `float64`). Defaults to auto select a minimal type that can represent the range of pixel values.
         scale_offset: bool, optional
             Whether to apply any EE band scales and offsets to the image.
+        bands: list of str, optional
+            List of band names to download.
         """
 
         max_threads = num_threads or min(32, (os.cpu_count() or 1) + 4)
@@ -837,7 +899,7 @@ class BaseImage:
         exp_image, profile = self._prepare_for_download(**kwargs)
 
         # get the dimensions of an image tile that will satisfy GEE download limits
-        tile_shape, num_tiles = self._get_tile_shape(exp_image, max_tile_size=max_tile_size, max_tile_dim=max_tile_dim)
+        tile_shape, num_tiles = exp_image._get_tile_shape(max_tile_size=max_tile_size, max_tile_dim=max_tile_dim)
 
         # find raw size of the download data (less than the actual download size as the image data is zipped in a
         # compressed geotiff)
@@ -846,16 +908,16 @@ class BaseImage:
             dtype_size = np.dtype(exp_image.dtype).itemsize
             raw_tile_size = tile_shape[0] * tile_shape[1] * exp_image.count * dtype_size
             logger.debug(f'{filename.name}:')
-            logger.debug(f'Uncompressed size: {self._str_format_size(raw_download_size)}')
+            logger.debug(f'Uncompressed size: {BaseImage._str_format_size(raw_download_size)}')
             logger.debug(f'Num. tiles: {num_tiles}')
             logger.debug(f'Tile shape: {tile_shape}')
-            logger.debug(f'Tile size: {self._str_format_size(int(raw_tile_size))}')
+            logger.debug(f'Tile size: {BaseImage._str_format_size(int(raw_tile_size))}')
 
         if raw_download_size > 1e9:
             # warn if the download is large (>1GB)
             logger.warning(
                 f'Consider adjusting `region`, `scale` and/or `dtype` to reduce the {filename.name}'
-                f' download size (raw: {self._str_format_size(raw_download_size)}).'
+                f' download size (raw: {BaseImage._str_format_size(raw_download_size)}).'
             )
 
         # configure the progress bar to monitor raw/uncompressed download size
@@ -869,7 +931,8 @@ class BaseImage:
 
         session = utils.retry_session(5)
         warnings.filterwarnings('ignore', category=TqdmWarning)
-        redir_tqdm = logging_redirect_tqdm([logging.getLogger(__package__)])  # redirect logging through tqdm
+        # redirect logging through tqdm
+        redir_tqdm = logging_redirect_tqdm([logging.getLogger(__package__)], tqdm_class=type(bar))
         env = rio.Env(GDAL_NUM_THREADS='ALL_CPUs', GTIFF_FORCE_RGBA=False)
         with redir_tqdm, env, rio.open(filename, 'w', **profile) as out_ds, bar:
 
@@ -881,7 +944,7 @@ class BaseImage:
 
             with ThreadPoolExecutor(max_workers=max_threads) as executor:
                 # Run the tile downloads in a thread pool
-                tiles = self._tiles(exp_image, tile_shape=tile_shape)
+                tiles = exp_image._tiles(tile_shape=tile_shape)
                 futures = [executor.submit(download_tile, tile) for tile in tiles]
                 try:
                     for future in as_completed(futures):
@@ -892,6 +955,8 @@ class BaseImage:
                     raise ex
 
             bar.update(bar.total - bar.n)   # ensure the bar reaches 100%
-            # populate GeoTIFF metadata and build overviews
-            self._write_metadata(out_ds)
-            self._build_overviews(out_ds)
+            # populate GeoTIFF metadata
+            exp_image._write_metadata(out_ds)
+
+        # build overviews
+        exp_image._build_overviews(filename)
