@@ -24,6 +24,7 @@ import rasterio as rio
 from rasterio import Affine, MemoryFile
 from rasterio.windows import Window
 from tqdm.auto import tqdm
+from geedim import utils
 
 
 class Tile:
@@ -100,7 +101,8 @@ class Tile:
                 ex_msg = f'Error downloading tile: {msg}'
                 if 'user memory limit exceeded' in msg.lower():
                     ex_msg += (
-                        '\nThe `max_tile_size` or `max_tile_dim` parameters can be decreased to work around this error.'
+                        '\nThe `max_tile_size` or `max_tile_dim` parameters can be decreased to work around this '
+                        'error.  Alternatively you can export to Earth Engine asset, and then download the asset image.'
                     )
             else:
                 ex_msg = str(response.json())
@@ -120,7 +122,8 @@ class Tile:
         ext_buffer = BytesIO(zip_file.read(zip_file.filelist[0]))
 
         # read the geotiff with a rasterio memory file
-        with rio.Env(GDAL_NUM_THREADS='ALL_CPUs', GTIFF_FORCE_RGBA=False), MemoryFile(ext_buffer) as mem_file:
+        env = rio.Env(GDAL_NUM_THREADS='ALL_CPUs', GTIFF_FORCE_RGBA=False)
+        with utils.suppress_rio_logs(), env, MemoryFile(ext_buffer) as mem_file:
             with mem_file.open() as ds:
                 array = ds.read()
                 if (array.dtype == np.dtype('float32')) or (array.dtype == np.dtype('float64')):
